#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <algorithm>
#include <numeric>

using namespace std;
using ll = long long;
using ld = long double;

struct GCDSegmentTree {
    int size{};
    vector<int> tree{};

    GCDSegmentTree(int size) : size(size) {
        tree.resize(size * 4);
    }

    void build(const vector<int>& initial_data) {
        do_build(0, 0, size, initial_data);
    }

    void do_build(int node, int low, int high, const vector<int>& initial_data) {
        if (low == high - 1) {
            tree[node] = initial_data[low];
            return;
        }

        int middle = (low + high) / 2;

        do_build(node * 2 + 1, low, middle, initial_data);
        do_build(node * 2 + 2, middle, high, initial_data);

        tree[node] = gcd(tree[node * 2 + 1], tree[node * 2 + 2]);
    }

    void set(int index, int value) {
        do_set(0, 0, size, index, value);
    }

    void do_set(int node, int low, int high, int index, int value) {
        if (low == high - 1 && low == index) {
            tree[node] = value;
            return;
        }

        int middle = (low + high) / 2;

        if (index < middle) {
            do_set(node * 2 + 1, low, middle, index, value);
        } else {
            do_set(node * 2 + 2, middle, high, index, value);
        }
    }

    int get(int low, int high) {
        return do_get(0, 0, size, low, high);
    }

    int do_get(int node, int low, int high, int query_low, int query_high) {
        if (low == query_low && high == query_high) {
            return tree[node];
        }

        if (query_low >= high || query_high <= low) {
            return 0;
        }

        int middle = (low + high) / 2;

        int left_result = do_get(node * 2 + 1, low, middle, query_low, min(query_high, middle));
        int right_result = do_get(node * 2 + 2, middle, high, max(query_low, middle), query_high);

        return gcd(left_result, right_result);
    }
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, q;
    cin >> n >> q;

    vector<int> array(n);

    for (auto& element : array) {
        cin >> element;
    }

    GCDSegmentTree gcd_tree(n);
    gcd_tree.build(array);

    for (int i = 0; i < q; i++) {
        int type;
        cin >> type;

        if (type == 1) {
            int x, v;
            cin >> x >> v;

            x -= 1;

            gcd_tree.set(x, v);
            array[x] = v;
        } else if (type == 2) {
            int low, high;
            cin >> low >> high;

            low -= 1;

            int inner_low = low;
            int inner_high = low;
            int curr_gcd = 0;
            ll result = 0;

            for (; inner_low < high; inner_low++) {
                inner_high = max(inner_low, inner_high);
                result += inner_high - inner_low;
                curr_gcd = inner_high == inner_low ? 0 : gcd_tree.get(inner_low, inner_high);

                while (inner_high < high) {
                    curr_gcd = gcd(curr_gcd, array[inner_high]);

                    if (curr_gcd == 1) {
                        break;
                    }

                    inner_high += 1;
                    result += 1;
                }
            }

            cout << result << "\n";
        }
    }

    return 0;
}
